import Flutter
import UIKit
import Smooch


public class SwiftSmoochPlugin: NSObject, FlutterPlugin {

    var channel : FlutterMethodChannel?

    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "smooch_plugin", binaryMessenger: registrar.messenger())

        let instance = SwiftSmoochPlugin()
        instance.channel = channel

        registrar.addApplicationDelegate(instance)
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {

        if call.method == "init" {
            initialize(call.arguments, result: result)
            return
        }

        if call.method == "show" {
            show(call.arguments, result: result)
            return
        }

        if call.method == "logout" {
            logout(result: result)
            return
        }

        if call.method == "minimize" {
            minimize(call.arguments, result: result)
            return
        }

        result(FlutterMethodNotImplemented)
    }

}

extension SwiftSmoochPlugin {

    // MARK: - FlutterApplicationLifeCycleDelegate
    public func application(_ application: UIApplication, didRegisterForRemoteNotificationsWithDeviceToken deviceToken: Data) {
        SmoochHandler.shared().registerPushToken(pushToken: deviceToken)
    }

    public func application(_ application: UIApplication, didReceiveRemoteNotification userInfo: [AnyHashable : Any], fetchCompletionHandler completionHandler: @escaping (UIBackgroundFetchResult) -> Void) -> Bool {
        SmoochHandler.handlePushNotification(userInfo: userInfo)
        channel?.invokeMethod("onPushNotification", arguments: userInfo)
        completionHandler(.noData)
        return true
    }

    // MARK: - UNUserNotificationCenterDelegate
    public func userNotificationCenter(_ center: UNUserNotificationCenter, willPresent notification: UNNotification, withCompletionHandler completionHandler: @escaping (UNNotificationPresentationOptions) -> Void) {
        NSLog("[SMOOCH] %@", #function)
    }

    public func userNotificationCenter(_ center: UNUserNotificationCenter, didReceive response: UNNotificationResponse, withCompletionHandler completionHandler: @escaping () -> Void) {
        NSLog("[SMOOCH] %@", #function)
    }

}

// MARK: - FlutterMethodCalls
extension SwiftSmoochPlugin {

    private func initialize(_ arguments: Any?, result: @escaping FlutterResult) {

        guard let arguments = arguments as? [String: Any] else {
            result(-1)
            return
        }

        if let properties = arguments["properties"] as? [String: Any] {
            SmoochHandler.shared().properties = properties
        }

        SmoochHandler.shared().setMenuItemsAllowed(arguments["menuItemsAllowed"])
        SmoochHandler.shared().setChatColor(arguments["chatColor"])

        if let appId = arguments["appId"] as? String,
            let userId = arguments["userId"] as? String,
            let smoochToken = arguments["smoochToken"] as? String {
            
            SmoochHandler.shared().app_id = appId
            SmoochHandler.shared().smooch_token = smoochToken

            let settings = SKTSettings(appId: appId)
            if let chatColor = SmoochHandler.shared().chatColor {
                settings.conversationAccentColor = chatColor
            }
            settings.conversationStatusBarStyle = .lightContent;
            settings.allowedMenuItems = SmoochHandler.shared().menuItensAllowed
            settings.enableAppDelegateSwizzling = false
            settings.enableUserNotificationCenterDelegateOverride = false

            Smooch.destroy()

            Smooch.initWith(settings) { error, dictionary in
                if let error = error {
                    print(error.localizedDescription)
                    result(-1)
                    return
                }
                let user = "\(userId)"
                Smooch.login(user, jwt: smoochToken, completionHandler: { error, info in

                    if let loginError = error {
                        print(loginError.localizedDescription)
                    }

                    if let loginInfo = info {
                        print(loginInfo)
                    }
                    
                    var user = SKTUser.current()
                    print("AppUer \(user?.userId)")
                    print("EMAIL \(user?.email)")
                    print("name \(user?.firstName)")
                    print("LastName \(user?.lastName)")
                    print("appUserId \(user?.appUserId)")

                    SmoochHandler.shared().smoochInited = true
                    SmoochHandler.shared().conversation = Smooch.conversation()
                    result(0)
                })
            }
        }
    }

    private func show(_ arguments: Any?, result: @escaping FlutterResult) {
        SmoochHandler.shared().show(channel) { (res) in
            result(res)
        }
    }

    private func logout(result: @escaping FlutterResult) {
        SmoochHandler.shared().logout { resultStatus in
            result(resultStatus)
        }
    }

    private func minimize(_ arguments: Any?, result: @escaping FlutterResult) {

        if let arguments = arguments {
            print("FlutterMethod \(#function) arguments: \(arguments)")
        }

        SmoochHandler.shared().closeConversarionViewController()
        result(nil)
    }

}
